#ifdef _WIN32
#include <windows.h>
#include <windowsx.h>
#include <time.h>
#else
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <dirent.h>
#include <time.h>
#endif

#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "math.h"

#include <FL/Fl.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Select_Browser.H>
#include <FL/Fl_Check_Browser.H>
#include <FL/Fl_Select_Browser.H>
#include <FL/Fl_Shared_Image.H>
#include <FL/Fl_Multiline_Input.H>

#include "List.h"

#include "cwindow.h"
#include "twindow.h"
#include "owindow.h"

#include "GMapTile.h"
#include "GMap.h"

int STARTX=20,STARTY=60;
int SX=640,SY=480;

bool quit=false;

// Application data: 
GMapsCWindow *cw=0;
GMapsTWindow *tw=0;
GMapsOWindow *ow=0;
// GMapsLWindow *lw=0;
int mode=0;	// 0 = nothing, 1 = tile, 2 = object 

GMap *map=0;
List<GMapTile> tiles;

char last_file_name[256]="";


int n_objects=36;
char *object_name[]={"GO_key","GO_fallingrock","GO_skull","GO_jumpingskull",
				     "GO_cagedoor","GO_dropgenerator","GO_skulldoor","GO_wateropening",
					 "GO_wateropening_blue","GO_wateropening_green",
					 "GO_entrydoor","GO_ropetop","GO_rope","GO_ropebottom",
					 "GO_water","GO_fallingrock_blue","GO_fallingrock_green","GO_bigrock",
					 "GO_bat","GO_skeleton","GO_fallingrock_yellow","GO_wateropening_yellow",
					 "GO_bigrock_blue","GO_bigrock_green","GO_bigrock_yellow",
					 "GO_pipe_water_left","GO_pipe_water_right","GO_trickyskull",
					 "GO_lava","GO_exitdoor","GO_closingwall","GO_closingwall_blue","GO_closingwall_green","GO_closingwall_yellow",
				  	 "GO_flame","GO_ghost"};
	
int object_parameters[]={0,0,1,1,
						 2,0,3,0,
						 0,0,
						 0,0,0,0,
						 0,0,0,0,
						 0,1,0,0,
						 0,0,0,
						 0,0,1,
						 0,0,0,0,0,0,
						 0,0};
char *object_parameter_name[36][3]={ {"","",""},{"","",""},{"facing","",""},{"facing","",""},
									{"n_locks","object",""},{"","",""},{"destination x","destination y","destination door"},{"","",""},
									{"","",""},{"","",""},
									{"","",""},{"","",""},{"","",""},{"","",""},
									{"","",""},{"","",""},{"","",""},{"","",""},
									{"","",""},{"facing","",""},{"","",""},{"","",""},
									{"","",""},{"","",""},{"","",""},
									{"","",""},{"","",""},{"facing","",""},
									{"","",""},{"","",""},{"","",""},{"","",""},{"","",""},{"","",""},
									{"","",""},{"","",""}};
int object_parameters_default[36][3]={{0,0,0},{0,0,0},{0,0,0},{0,0,0},
									{1,0,0},{0,0,0},{0,0,0},{0,0,0},
									{0,0,0},{0,0,0},
									{0,0,0},{0,0,0},{0,0,0},{0,0,0},
									{0,0,0},{0,0,0},{0,0,0},{0,0,0},
									{0,0,0},{0,0,0},{0,0,0},{0,0,0},
									{0,0,0},{0,0,0},{0,0,0},
									{0,0,0},{0,0,0},{0,0,0},
									{0,0,0},{0,0,0},{0,0,0},{0,0,0},{0,0,0},
									{0,0,0},{0,0,0}};


// auxiliary functions:
void search_tiles(char *path);

#ifndef _WIN32
 #ifdef __APPLE__
int filter_png(struct dirent *de) {
 #else
int filter_png(const struct dirent *de) {
 #endif //__APPLE__
	if (strcmp(".png", de->d_name+strlen(de->d_name)-4) == 0) return(1);
	else return(0);
}
#endif //_WIN32


int main(int argc,char **argv)
{
	Fl::visual(FL_DOUBLE|FL_RGB);

	fl_register_images();

	search_tiles("graphics");
	map=new GMap(32*16,20*16);

	cw=new GMapsCWindow(5,5);
//	lw=new GMapsLWindow(145+520,5);
	cw->show(argc,argv);
//	lw->show(argc,argv);
	
	while(!quit) {
		Fl::check();
		cw->redraw();
#ifdef _WIN32
		Sleep(1);
#else
		usleep(10000);
#endif //_WIN32
	} // while

	delete cw;
//	delete lw;
	if (tw!=0) delete tw;
	if (ow!=0) delete ow;

	return 0;
} /* main */ 



void search_tiles(char *path)
{
	
#ifdef _WIN32
	/* Find files: */ 
	WIN32_FIND_DATA finfo;
	HANDLE h;
	char pattern[256],tmp[256];
	sprintf(pattern,"%s/*.*",path);

	h=FindFirstFile(pattern,&finfo);
	if (h!=INVALID_HANDLE_VALUE) {
		strcpy(tmp,finfo.cFileName);
		while(FindNextFile(h,&finfo)==TRUE) {
			if (finfo.cFileName[strlen(finfo.cFileName)-1]=='g' &&
				finfo.cFileName[strlen(finfo.cFileName)-2]=='n' &&
				finfo.cFileName[strlen(finfo.cFileName)-3]=='p' &&
				finfo.cFileName[strlen(finfo.cFileName)-4]=='.') {
				sprintf(tmp,"%s/%s",path,finfo.cFileName);
				tiles.Add(new GMapTile(tmp));
			} /* if */ 
		} /* while */ 
	} /* if */ 
#else
	struct dirent **namelist;
	char tmp[512];
	int i,n;

	// get alpha sorted list of ".png" filenames from "path"
	if ((n=scandir(path, &namelist, filter_png, alphasort)) > 0) {
		for (i=0; i<n; i++) {
			snprintf(tmp, 512, "%s/%s", path, namelist[i]->d_name);
			tiles.Add(new GMapTile(tmp));
		}
		free(namelist);
	}
#endif  

} /* search_tiles */ 

